@@ -12,21 +12,28 @@ BUFFER_SIZE = 3 * 1024 * 1024  # 3MB TODO: Base this on some grpc value
 
 
 class PfsClient(object):
-    def __init__(self, host=None, port=None, auth_token=None):
+    def __init__(self, host=None, port=None, auth_token=None, root_certs=None):
         """
         Creates a client to connect to PFS.
 
         Params:
+
         * host: The pachd host. Default is 'localhost', which is used with
         `pachctl port-forward`.
         * port: The port to connect to. Default is 30650.
         * auth_token: The authentication token; used if authentication is
         enabled on the cluster. Default to `None`.
+        * root_certs:  The PEM-encoded root certificates as byte string.
         """
 
         address = get_address(host, port)
         self.metadata = get_metadata(auth_token)
-        self.channel = grpc.grpc.insecure_channel(address)
+        if root_certs:
+            ssl_channel_credentials = grpc.grpc.ssl_channel_credentials
+            ssl = ssl_channel_credentials(root_certificates=root_certs)
+            self.channel = grpc.grpc.secure_channel(address, ssl)
+        else:
+            self.channel = grpc.grpc.insecure_channel(address)
         self.stub = grpc.APIStub(self.channel)
 
     def create_repo(self, repo_name, description=None, update=None):
@@ -38,6 +45,7 @@ class PfsClient(object):
         database dumps etc.
 
         Params:
+
         * repo_name: Name of the repo.
         * description: An optional string describing the repo.
         * update: Whether to update if the repo already exists.
@@ -72,6 +80,7 @@ class PfsClient(object):
         Deletes a repo and reclaims the storage space it was using.
 
         Params:
+
         * repo_name: The name of the repo.
         * force: If set to true, the repo will be removed regardless of
         errors. This argument should be used with care.
@@ -84,6 +93,7 @@ class PfsClient(object):
         Deletes all repos.
 
         Params:
+
         * force: If set to true, the repo will be removed regardless of
         errors. This argument should be used with care.
         """
@@ -100,6 +110,7 @@ class PfsClient(object):
         returned.
 
         Params:
+
         * repo_name: A string specifying the name of the repo.
         * branch: A string specifying the branch name. This is a more
         convenient way to build linear chains of commits. When a commit is
@@ -132,6 +143,7 @@ class PfsClient(object):
         future attempts to write to it with PutFile will error.
 
         Params:
+
         * commit: A tuple, string, or `Commit` object representing the commit.
         * description: An optional string describing this commit.
         * tree_object_hashes: A list of zero or more strings specifying object
@@ -158,6 +170,7 @@ class PfsClient(object):
         A context manager for running operations within a commit.
 
         Params:
+
         * repo_name: A string specifying the name of the repo.
         * branch: A string specifying the branch name. This is a more
         convenient way to build linear chains of commits. When a commit is
@@ -182,6 +195,7 @@ class PfsClient(object):
         Inspects a commit. Returns a `CommitInfo` object.
 
         Params:
+
         * commit: A tuple, string, or `Commit` object representing the commit.
         * block_state: Causes inspect commit to block until the commit is in
         the desired commit state.
@@ -194,6 +208,7 @@ class PfsClient(object):
         Lists commits. Yields `CommitInfo` objects.
 
         Params:
+
         * repo_name: If only `repo_name` is given, all commits in the repo are
         returned.
         * to_commit: Optional. Only the ancestors of `to`, including `to`
@@ -216,6 +231,7 @@ class PfsClient(object):
         Deletes a commit.
 
         Params:
+
         * commit: A tuple, string, or `Commit` object representing the commit.
         """
         req = proto.DeleteCommitRequest(commit=commit_from(commit))
@@ -237,6 +253,7 @@ class PfsClient(object):
         Yields `CommitInfo` objects.
 
         Params:
+
         * commits: A list of tuples, strings, or `Commit` objects representing
         the commits to flush.
         * repos: An optional list of strings specifying repo names. If
@@ -252,6 +269,7 @@ class PfsClient(object):
         Yields `CommitInfo` objects as commits occur.
 
         Params:
+
         * repo_name: A string specifying the name of the repo.
         * branch: A string specifying branch to subscribe to.
         * from_commit_id: An optional string specifying the commit ID. Only
@@ -297,6 +315,7 @@ class PfsClient(object):
         `BranchInfo` objects.
 
         Params:
+
         * repo_name: A string specifying the repo name.
         """
         req = proto.ListBranchRequest(repo=proto.Repo(name=repo_name))
@@ -310,6 +329,7 @@ class PfsClient(object):
         branches they happen to be on.
 
         Params:
+
         * repo_name: A string specifying the repo name.
         * branch_name: A string specifying the name of the branch to delete.
         * force: A bool specifying whether to force the branch deletion.
@@ -324,6 +344,7 @@ class PfsClient(object):
         Uploads a binary bytes array as file(s) in a certain path.
 
         Params:
+
         * commit: A tuple, string, or `Commit` object representing the commit.
         * path: A string specifying the path in the repo the file(s) will be
         written to.
@@ -396,24 +417,32 @@ class PfsClient(object):
 
         self.stub.PutFile(wrap(value), metadata=self.metadata)
 
-    def put_file_url(self, commit, path, url, recursive=None):
+    def put_file_url(self, commit, path, url, recursive=None, overwrite_index=None):
         """
         Puts a file using the content found at a URL. The URL is sent to the
         server which performs the request. Note that this is not a standard
         PFS function.
 
         Params:
+
         * commit: A tuple, string, or `Commit` object representing the commit.
         * path: A string specifying the path to the file.
         * url: A string specifying the url of the file to put.
         * recursive: allow for recursive scraping of some types URLs, for
         example on s3:// URLs.
+        * overwrite_index: An optional `OverwriteIndex` object. This is the
+        object index where the write starts from.  All existing objects
+        starting from the index are deleted.
         """
+
+        overwrite_index_proto = proto.OverwriteIndex(index=overwrite_index) if overwrite_index is not None else None
+
         req = iter([
             proto.PutFileRequest(
                 file=proto.File(commit=commit_from(commit), path=path),
                 url=url,
-                recursive=recursive
+                recursive=recursive,
+                overwrite_index=overwrite_index_proto
             )
         ])
         self.stub.PutFile(req, metadata=self.metadata)
@@ -425,6 +454,7 @@ class PfsClient(object):
         1.9.0) silently fail.
 
         Params:
+
         * source_commit: A tuple, string, or `Commit` object representing the
         commit for the source file.
         * source_path: A string specifying the path of the source file.
@@ -446,6 +476,7 @@ class PfsClient(object):
         Returns an iterator of the contents of a file at a specific commit.
 
         Params:
+
         * commit: A tuple, string, or `Commit` object representing the commit.
         * path: A string specifying the path of the file.
         * offset_bytes: An optional int. Specifies a number of bytes that
@@ -469,6 +500,7 @@ class PfsClient(object):
         Inspects a file. Returns a `FileInfo` object.
 
         Params:
+
         * commit: A tuple, string, or `Commit` object representing the commit.
         * path: A string specifying the path to the file.
         """
@@ -480,6 +512,7 @@ class PfsClient(object):
         Lists the files in a directory.
 
         Params:
+
         * commit: A tuple, string, or `Commit` object representing the commit.
         * path: The path to the directory.
         * history: An optional int that indicates to return jobs from
@@ -506,6 +539,7 @@ class PfsClient(object):
         `FileInfo` objects.
 
         Params:
+
         * commit: A tuple, string, or `Commit` object representing the commit.
         * path: The path to the directory.
         """
@@ -519,6 +553,7 @@ class PfsClient(object):
         Lists files that match a glob pattern. Yields `FileInfo` objects.
 
         Params:
+
         * commit: A tuple, string, or `Commit` object representing the commit.
         * pattern: A string representing a glob pattern.
         """
@@ -534,6 +569,7 @@ class PfsClient(object):
         will of course remain intact in the Commit's parent.
 
         Params:
+
         * commit: A tuple, string, or `Commit` object representing the commit.
         * path: The path to the file.
         """
